% Program for calculating the exposed length of palisade and spongy
% mesophyll, area and aspect ratio of the cells

clear all
currentFolder=pwd;
result_tot=[];
result_tot2=[];

for z=1:length(p_h_2_w)


load cord   %% coordinates of the vertexId for each of the tissues
load CellId %% contains the cellvertexId of each vertex 
load PID    %% information about which cell is a palisade in each tissue
load EID    %% information about which cell is a epidermal cell in each tissue
load WALL   %% information about which vertices defines a wall
load W2C    %% information about which wall belongs to a cell

palisade_wall_exposed_length=[];
spongy_wall_exposed_length=[];

result=[];  %% Matrix containing the results

for zz=1:length(CellId)  % For each geometry
 % Initialization   
 coordinates=[];
 CellvertexId=[];   
 epiId=[];          
 paliId=[];
 walls=[];
 wall2cell=[];
 spongyvertex=[];
 spongyId=[];
 V_walls=[];        % Will contain all vertices that contains internal boundaries of pallisade parenchyma 
 
 % Assign values to initialized vectors
 coordinates=cord{zz};
 CellvertexId=CellId{zz};
 epiId=EID{zz};
 paliId=PID{zz};
 walls=WALL{zz};
 wall2cell=W2C{zz};

%% to calculate exposed palisade to to total length

 spongyId=find(~ismember([1:length(CellvertexId)],[paliId;epiId]));

for i=1:length(walls) % For each cell wall
    if ((ismember(wall2cell(i,1),paliId)& ~ismember(wall2cell(i,2),paliId))...
        |(ismember(wall2cell(i,2),paliId)& ~ismember(wall2cell(i,1),paliId)))...
        & (~ismember(wall2cell(i,1),[spongyId';epiId])&~ismember(wall2cell(i,2),[spongyId';epiId]))
    palisade_wall_exposed_length=[palisade_wall_exposed_length;norm(coordinates(walls(i,1),:)-coordinates(walls(i,2),:),2)];
   
    end
end

EPL2LW=sum(palisade_wall_exposed_length)/(max(coordinates(:,1))-min(coordinates(:,1)));

for i=1:length(walls) % For each cell wall
    if ((ismember(wall2cell(i,1),spongyId)& ~ismember(wall2cell(i,2),spongyId))...
        |(ismember(wall2cell(i,2),spongyId)& ~ismember(wall2cell(i,1),spongyId)))...
        & (~ismember(wall2cell(i,1),[paliId;epiId])&~ismember(wall2cell(i,2),[paliId;epiId]))
    spongy_wall_exposed_length=[spongy_wall_exposed_length;norm(coordinates(walls(i,1),:)-coordinates(walls(i,2),:),2)];
   
    end
end

ESL2LW=sum(spongy_wall_exposed_length)/(max(coordinates(:,1))-min(coordinates(:,1)));
total=(sum(spongy_wall_exposed_length)+sum(palisade_wall_exposed_length))/(max(coordinates(:,1))-min(coordinates(:,1)));

Leaf_width=max(coordinates(:,1))-min(coordinates(:,1));
Thickness=max(coordinates(:,2))-min(coordinates(:,2));

counter1=0;
counter2=0;
counter3=0;

for k=1:length(CellvertexId),
if ismember (k,epiId)
    counter1=counter1+1;
  patch(coordinates(CellvertexId{k},1),coordinates(CellvertexId{k},2),'r')
    hold on
  text(mean(coordinates(CellvertexId{k},1)),mean(coordinates(CellvertexId{k},2)),[[strcat(num2str(k))]],'FontSize',10);
  A_1_epi(counter1)=polyarea(coordinates(CellvertexId{k},1),coordinates(CellvertexId{k},2));
  xx=coordinates(CellvertexId{k}(1:length(CellvertexId{k}(:))-1),1);
    yy=coordinates(CellvertexId{k}(1:length(CellvertexId{k}(:))-1),2);
    
    x2 = [xx(2:length(xx),1);xx(1,1)];
    y2 = [yy(2:length(yy),1);yy(1,1)];
    %% calculating second moments of inertia
a = (1/2*sum(x2.*yy-xx.*y2));
p=sum(sqrt((x2-xx).*(x2-xx)+(y2-yy).*(y2-yy)));

x0(k) = 1/6*sum((x2.*yy-xx.*y2).*(xx+x2))/a;
y0(k) = 1/6*sum((x2.*yy-xx.*y2).*(yy+y2))/a;
Ixx= sum((yy.*yy +yy.*y2+y2.*y2).*(x2.*yy-xx.*y2))/(12);
Ixy= sum((2*xx.*yy +xx.*y2+x2.*yy+2*x2.*y2).*(x2.*yy-xx.*y2))/(24);
Iyy= sum((xx.*xx +xx.*x2+x2.*x2).*(x2.*yy-xx.*y2))/(12);

%% calculating  centroidal moments

u20=Ixx-a*(y0(k)*y0(k));

u02=Iyy-a*(x0(k)*x0(k));

u11=Ixy-a*(x0(k)*y0(k));

%% calculating major and minor diameter of a hypothetical ellipse that is
% the best fit to the set of points

L1=sqrt((2*(u20+u02+sqrt(4*u11*u11+(u02-u20)*(u02-u20)))/u11));
L2=sqrt((2*(u20+u02-sqrt(4*u11*u11+(u02-u20)*(u02-u20)))/u11));

%% calculating the dominant orientation

beta(k)=1/2*atan(2*u11/(u02-u20))*180/pi;
orientation(k)=beta(k);

%% calculating the aspect ratio

AR(k)=1-min(L2/L1,L1/L2);
Aspectratio_1_epi(counter1)=AR(k);

elseif ismember(k,paliId)
    counter2=counter2+1
  patch(coordinates(CellvertexId{k},1),coordinates(CellvertexId{k},2),'m')
    hold on
  text(mean(coordinates(CellvertexId{k},1)),mean(coordinates(CellvertexId{k},2)),[[strcat(num2str(k))]],'FontSize',10);
  A_1_pali(counter2)=polyarea(coordinates(CellvertexId{k},1),coordinates(CellvertexId{k},2));
  
  xx=coordinates(CellvertexId{k}(1:length(CellvertexId{k}(:))-1),1);
    yy=coordinates(CellvertexId{k}(1:length(CellvertexId{k}(:))-1),2);
    
    x2 = [xx(2:length(xx),1);xx(1,1)];
    y2 = [yy(2:length(yy),1);yy(1,1)];
    %% calculating second moments of inertia
a = (1/2*sum(x2.*yy-xx.*y2));
p=sum(sqrt((x2-xx).*(x2-xx)+(y2-yy).*(y2-yy)));

x0(k) = 1/6*sum((x2.*yy-xx.*y2).*(xx+x2))/a;
y0(k) = 1/6*sum((x2.*yy-xx.*y2).*(yy+y2))/a;
Ixx= sum((yy.*yy +yy.*y2+y2.*y2).*(x2.*yy-xx.*y2))/(12);
Ixy= sum((2*xx.*yy +xx.*y2+x2.*yy+2*x2.*y2).*(x2.*yy-xx.*y2))/(24);
Iyy= sum((xx.*xx +xx.*x2+x2.*x2).*(x2.*yy-xx.*y2))/(12);

%% calculating  centroidal moments

u20=Ixx-a*(y0(k)*y0(k));

u02=Iyy-a*(x0(k)*x0(k));

u11=Ixy-a*(x0(k)*y0(k));

%% calculating major and minor diameter of a hypothetical ellipse that is
% the best fit to the set of points

L1=sqrt((2*(u20+u02+sqrt(4*u11*u11+(u02-u20)*(u02-u20)))/u11));
L2=sqrt((2*(u20+u02-sqrt(4*u11*u11+(u02-u20)*(u02-u20)))/u11));

%% calculating the dominant orientation

beta(k)=1/2*atan(2*u11/(u02-u20))*180/pi;
orientation(k)=beta(k);

%% calculating the aspect ratio

AR(k)=1-min(L2/L1,L1/L2);

Aspectratio_1_pali(counter2)=AR(k); 
else
    counter3=counter3+1;
   patch(coordinates(CellvertexId{k},1),coordinates(CellvertexId{k},2),'g')
    hold on
  text(mean(coordinates(CellvertexId{k},1)),mean(coordinates(CellvertexId{k},2)),[[strcat(num2str(k))]],'FontSize',10);
  A_1_spongy(counter3)=polyarea(coordinates(CellvertexId{k},1),coordinates(CellvertexId{k},2));
  
   xx=coordinates(CellvertexId{k}(1:length(CellvertexId{k}(:))-1),1);
    yy=coordinates(CellvertexId{k}(1:length(CellvertexId{k}(:))-1),2);
    
    x2 = [xx(2:length(xx),1);xx(1,1)];
    y2 = [yy(2:length(yy),1);yy(1,1)];
    %% calculating second moments of inertia
a = (1/2*sum(x2.*yy-xx.*y2));
p=sum(sqrt((x2-xx).*(x2-xx)+(y2-yy).*(y2-yy)));

x0(k) = 1/6*sum((x2.*yy-xx.*y2).*(xx+x2))/a;
y0(k) = 1/6*sum((x2.*yy-xx.*y2).*(yy+y2))/a;
Ixx= sum((yy.*yy +yy.*y2+y2.*y2).*(x2.*yy-xx.*y2))/(12);
Ixy= sum((2*xx.*yy +xx.*y2+x2.*yy+2*x2.*y2).*(x2.*yy-xx.*y2))/(24);
Iyy= sum((xx.*xx +xx.*x2+x2.*x2).*(x2.*yy-xx.*y2))/(12);

%% calculating  centroidal moments

u20=Ixx-a*(y0(k)*y0(k));

u02=Iyy-a*(x0(k)*x0(k));

u11=Ixy-a*(x0(k)*y0(k));

%% calculating major and minor diameter of a hypothetical ellipse that is
% the best fit to the set of points

L1=sqrt((2*(u20+u02+sqrt(4*u11*u11+(u02-u20)*(u02-u20)))/u11));
L2=sqrt((2*(u20+u02-sqrt(4*u11*u11+(u02-u20)*(u02-u20)))/u11));


%% calculating the dominant orientation

beta(k)=1/2*atan(2*u11/(u02-u20))*180/pi;
orientation(k)=beta(k);

%% calculating the aspect ratio

AR(k)=1-min(L2/L1,L1/L2);

Aspectratio_1_spongy(counter3)=AR(k);   

end
end


A_epi_1=([A_1_epi]);
A_pali_6=([A_1_pali]);
A_spongy_6=([A_1_spongy]);
Mean_A_e= mean(A_epi_1);
std_A_e=std(A_epi_1);
Mean_A_p= mean(A_pali_6);
std_A_p=std(A_pali_6);
Mean_A_s= mean(A_spongy_6);
std_A_s=std(A_spongy_6);

Mean_A_p= mean(A_pali_6);
Mean_A_s= mean(A_spongy_6);

aspect_epi=[Aspectratio_1_epi];
aspect_pali_6=[Aspectratio_1_pali];
aspect_spongy_6=[Aspectratio_1_spongy];
Mean_As_p=mean(aspect_pali_6);
Std_As_p=std(aspect_pali_6);
Mean_As_S=mean(aspect_spongy_6);
Std_As_s=std(aspect_spongy_6);
Mean_As_e=mean(aspect_epi);
Std_As_e=std(aspect_epi);


% mean values
result=[result;EPL2LW ESL2LW total Mean_A_e std_A_e Mean_A_p std_A_p Mean_A_s std_A_s Mean_As_e Std_As_e Mean_As_p Std_As_p Mean_As_S Std_As_s Thickness Leaf_width];


end
result_tot=[result_tot;result];

end




