% This code solves the growth algorithm equations using ode15s solver after
% the initial voronoi tesselations are generated. The doublewalls function
% generates the intitial tesselation, defines wall2vertex relations,
% wall2cell relations and cell2vertex relations

function [coordinates,dimension,numberofcells,walls,wall2cell,l_n,Cellarea,CellvertexId,t,l_n_0,y,epiId,paliId,hypoId,KK,geom]=GROWTH(ap,ar,dr,l_n_max_2_l_n_0,P_h_2_w,geom,numcell,string,string11)

[coordinates,dimension,numberofcells,walls,wall2cell,Wallbnd,l_n,Cellarea,CellvertexId,epiId,hypoId,paliId,stoneId,wallvepiuper,oldwall,adhesive_wall,adw,WO_1,WO_2,...
ad_direction,old_cord,split_wall,split_turgor_1,split_turgor_2,con_point,con_point_2,con_point_3]=doublewalls(geom,numcell,string, string11);

%% Parameters
E=30;        % Young's modulus (MPa)
tau1=200000; % Time constant for length to grow to maximum

%% assigning turgore pressure to the cells, spring constant to the walls  and damping coefficient for the matrix

turgorpressure(numberofcells,1)=0;
A0=mean(Cellarea);                 % average cell area of the initial cells
drag=dr;                           % drag coefficient 
l_n_0=l_n;                         % initial resting length
l_nmax=l_n_max_2_l_n_0*l_n_0;      % setting the maximum resting length of the walls

%% initializing the variables
amk=[];  % The walls of palisade cells that are parallel to growth direction
akk=[];  % The walls of palisade cells that are perpendicular to growth direction
aml=[];  % The walls of palisade cells that are in the initial tesselatin

anisotropy=zeros(length(walls),1);

for i=1:length(walls)

    ak(i)=E/mean(l_n_0(i));%(i);
    C_l_n(i)=l_n_max_2_l_n_0;

      if (ismember(wall2cell(i,1),paliId)&ismember(wall2cell(i,2),paliId))|((wall2cell(i,1)==numberofcells&ismember(wall2cell(i,2),paliId)...
              &abs(coordinates(walls(i,1),1)-coordinates(walls(i,2),1))<0.00001)...
              |(wall2cell(i,2)==numberofcells&ismember(wall2cell(i,1),paliId)&abs(coordinates(walls(i,1),1)-coordinates(walls(i,2),1))<0.00001))
                anisotropy(i)=1;
                C_l_n(i)=P_h_2_w*l_n_max_2_l_n_0; %setting the maximum resting length of the walls
                amk=[amk,i];
       elseif (ismember(wall2cell(i,1),epiId)|ismember(wall2cell(i,2),epiId))...
                ak(i)=ak(i);
       end
end
   

for i=1:length(walls)
    if (ismember(wall2cell(i,1),paliId)|ismember(wall2cell(i,2),paliId))& ~ismember(i,amk)
        akk=[akk,i];
        ak(i)=6*ak(i);
    end
end

for i=1:length(walls)
    if ~ismember(i,[akk,amk]) 
        ak(i)=2*ak(i);
    end
end


for i=1:oldwall
    if (ismember(wall2cell(i,1),paliId)|ismember(wall2cell(i,2),paliId))& ~ismember(i,amk)
       aml=[aml,i];
       l_n_0(i)=1*l_n_0(i);
    end
end
   
for i=1:length(walls)
    
Degree1(i)= atan((coordinates(walls(i,2),2)-coordinates(walls(i,1),2))/(coordinates(walls(i,2),1)-coordinates(walls(i,1),1)));
lambda(i)=1-(1-anisotropy(i)*(cos(Degree1(i)+pi/2))^2);
l_nmax(i)=lambda(i)*(C_l_n(i)-1)*l_n_0(i)+C_l_n(i)*l_n_0(i);  
end

tau=tau1;
%% Solving the growth equations
velocity=zeros(length(coordinates),dimension);
tstart=0;
tfinal=5000000;
at=10^(-ar);    % absolute tolerance
rt=10^(-(ar+1));%relative tolerance
pr=0;           % initialing turgore pressure
tc=100000;      % time constant for torgore pressure to reach maximum
pmax=ap;
y0=[coordinates(:,1);coordinates(:,2);l_n;pr];% intial conditions for the ode solver

options = odeset('RelTol',rt,'AbsTol',at,'events',@events,'stats','on','MaxOrder',1,'BDF','on');

%% beginning of the solution
for j=1:1
%% identifing walls of boundary cell and assigning spring constant
for i=1:length(walls)
    if  ismember(i, amk)
       KK(i)=ak(i)*(C_l_n(i)-1)*(1-lambda(i))+ak(i);
    else   
        KK(i)=ak(i);
    end
end 

left_boundary=[];
right_boundary=[];
bottom_boundaryy=[];
top_boundaryy=[];

% Finding boundaries of the geometry
for i=1:length(coordinates)
    if abs(coordinates(i,1)-min(coordinates(:,1)))<0.00001
        left_boundary=[left_boundary,i];
    end
    if abs(coordinates(i,1)-max(coordinates(:,1)))<0.00001
        right_boundary=[right_boundary,i];
    end

    if abs(coordinates(i,2)-min(coordinates(:,2)))<0.00001
        bottom_boundaryy=[bottom_boundaryy,i];
    end
    if abs(coordinates(i,2)-max(coordinates(:,2)))<0.00001
        top_boundaryy=[top_boundaryy,i];
    end

end

Ab_bottom=bottom_boundaryy(find(bottom_boundaryy<old_cord+1));
Ab_top=top_boundaryy(find(top_boundaryy<old_cord+1));

az_top=[Ab_top(1:2:end),top_boundaryy(find(top_boundaryy>old_cord))];
az_bottom=[Ab_bottom(1:2:end),bottom_boundaryy(find(bottom_boundaryy>old_cord))];
az_top_2=[Ab_top(2:2:end)];
az_bottom_2=[Ab_bottom(2:2:end)];
con_left=coordinates(left_boundary,1);
con_b1=coordinates(az_bottom,2);
con_b2=coordinates(az_bottom_2,2);

bottom_left=find(abs(coordinates(:,1)-min(coordinates(:,1)))<0.0001&abs(coordinates(:,2)-min(coordinates(:,2)))<0.0001);
bottom_right=find(abs(coordinates(:,1)-max(coordinates(:,1)))<0.0001&abs(coordinates(:,2)-min(coordinates(:,2)))<0.0001);
top_left=find(abs(coordinates(:,1)-min(coordinates(:,1)))<0.0001&abs(coordinates(:,2)-max(coordinates(:,2)))<0.0001);
top_right=find(abs(coordinates(:,1)-max(coordinates(:,1)))<0.0001&abs(coordinates(:,2)-max(coordinates(:,2)))<0.0001);
top_boundary=top_boundaryy;
bottom_boundary=bottom_boundaryy;
con_bottom=coordinates(bottom_boundary,2);

ABD=1:length(coordinates);
stopc=ABD(find(~ismember(ABD,[left_boundary,right_boundary,bottom_boundary,top_boundary])));

for i=1:length(walls)
    if (ismember(wall2cell(i,1),paliId)& ~ismember(wall2cell(i,2),paliId))|(ismember(wall2cell(i,2),paliId)& ~ismember(wall2cell(i,1),paliId));%,[225,246,247,252])
             
    lambda(i)=1;
    l_nmax(i)=lambda(i)*(C_l_n(i)-1)*l_n_0(i)+C_l_n(i)*l_n_0(i);  
              
    end
end
 

savetime=tstart:5000:tfinal;

[t,y]=ode15s(@f,[savetime],y0,options); % the ODE solver

end

%% defining the ODE function to be solved

function dydt=f(t,y)
    
   %% initializing the parameters at every time step
 b=[];
 force=zeros(length(coordinates),dimension);
 pf=zeros(length(coordinates),dimension);
 Tf=zeros(length(coordinates),dimension);

    coordinates=[y(1:1*length(coordinates)),y(1*length(coordinates)+1:2*length(coordinates))];% new coordinates
    coordinates(left_boundary,1)=con_left;
    velocity(left_boundary,1)=mean(velocity(left_boundary,1));
    velocity(right_boundary,1)=mean(velocity(right_boundary,1));

    velocity(az_top_2,2)=mean(velocity(az_top_2,2));
    velocity(az_bottom,2)=mean(velocity(az_bottom,2));
    velocity(az_top,2)=mean(velocity(az_top,2));
    velocity(az_bottom_2,2)=mean(velocity(az_bottom_2,2));
    velocity(az_bottom_2,2)=mean(velocity(az_bottom_2,2));

    l_n=y(2*length(coordinates)+1:length(y)-1);
    turgorpressure(1:numberofcells-1,1)=y(end);


%Calculate new Length and direction of walls

for i=1:length(walls)
    l(i)=norm(coordinates(walls(i,1),:)-coordinates(walls(i,2),:),2);
        direction(i,:)=(coordinates(walls(i,1),:)-coordinates(walls(i,2),:))/l(i);
        normal_direction(i,1)=direction(i,2);
        normal_direction(i,2)=-direction(i,1);
    Degree1(i)= atan((coordinates(walls(i,2),2)-coordinates(walls(i,1),2))/(coordinates(walls(i,2),1)-coordinates(walls(i,1),1)));

    Fturgor=(turgorpressure(wall2cell(i,2))-turgorpressure(wall2cell(i,1)))*l(i)*(normal_direction(i,:));
      
%% tension force  

      b(i)=((l(i)-l_n(i)).*(l_nmax(i)-l_n(i)))/(tau);

          Ftension=KK(i)*(l(i)-l_n(i))*direction(i,:);
             
 %% net force

       pf(walls(i,1),:)=pf(walls(i,1),:)+0.5*Fturgor; 
       pf(walls(i,2),:)=pf(walls(i,2),:)+0.5*Fturgor ;
       Tf(walls(i,1),:)=Tf(walls(i,1),:)-Ftension; 
       Tf(walls(i,2),:)=Tf(walls(i,2),:)+Ftension;

    kr(i)=0;

end

for i=1:length(adhesive_wall)
     ll(i)=1+norm(coordinates(adhesive_wall(i,1),:)-coordinates(adhesive_wall(i,2),:),2);
     Tf(adhesive_wall(i,1),:)=Tf(adhesive_wall(i,1),:)-100*(coordinates(adhesive_wall(i,1),:)-coordinates(adhesive_wall(i,2),:))/ll(i); 
     Tf(adhesive_wall(i,2),:)=Tf(adhesive_wall(i,2),:)+100*(coordinates(adhesive_wall(i,1),:)-coordinates(adhesive_wall(i,2),:))/ll(i);
          
end

force=pf+Tf;
force(left_boundary,1)=0;
force(az_bottom,2)=0;
 
velocity(:,1)=force(:,1)/drag;
velocity(:,2)=force(:,2)/drag;
      
      
velocity(left_boundary,1)=mean(velocity(left_boundary,1));
velocity(right_boundary,1)=mean(velocity(right_boundary,1));

velocity(az_top_2,2)=mean(velocity(az_top_2,2));
velocity(az_bottom,2)=mean(velocity(az_bottom,2));
velocity(az_top,2)=mean(velocity(az_top,2));
velocity(az_bottom_2,2)=mean(velocity(az_bottom_2,2));
velocity(az_bottom_2,2)=mean(velocity(az_bottom_2,2));

    p=1/tc*(pmax-y(end));

    dydt=[velocity(:,1);velocity(:,2);b(:);p];
    
end
 %% saving the time history of forces on vertices
function [value,isterminal,direction] = events(t,y)
   for i=1:length( CellvertexId) 
     Cellarea(i)=polyarea(coordinates(CellvertexId{i}(:),1),coordinates(CellvertexId{i}(:),2));
   end

areacheck=(mean(Cellarea)-5*A0);
NV=norm(velocity(stopc,:));
value=mean(NV)-0.000001;   % checking equilibrium velocities
t1=t;

isterminal = ones(1,1);
direction =-ones(1,1);
end  

%% to be saved
cd(string)

save coordinates coordinates
save CellvertexId CellvertexId
save walls walls
save wall2cell wall2cell
save l_n l_n
save epiId epiId
save paliId paliId
save KK KK
save adhesive_wall adhesive_wall
save split_wall split_wall
save split_turgor_1 split_turgor_1
save split_turgor_2 split_turgor_2
save left_boundary left_boundary
save right_boundary right_boundary
save az_top az_top
save az_top_2 az_top_2
save az_bottom az_bottom
save az_bottom_2 az_bottom_2
save con_point con_point
save con_point_2 con_point_2
save con_point_3 con_point_2
cd(string11)

 

end
