% Main program of tissue growth algorithm
% The algorithm is the extension of fruit growth model presented in 
% Abera et al. 2013. Food and Bioprocess Technology.(6),4.pp.859-869
% The first step is to run the case GROWTH. The coordinates  and vertices
% of the generated gometries  and are stored in the cords folder along with
% other outputs essential for splitting phase. Next, the case 'SPLITING' is
% run and the associated vertices are stored in split folder.

%% Input parameters
ap=1;               % Turgor pressure (Mpa)                                                                                                                                                                                                           
ar=1;               % The absolute tolerance of the solver is 10^-ar 
dr=100;             % Damping coefficient(-)
l_n_max_2_l_n_0=1;  % Ratio of maximum resting length and initial resting length of walls
P_h_2_w=5.0;        % Palisade height to width ratio
n=3;                % Number of geometries you want to generate
numcell=250;        % Number of cells in a geometry

steps= 'SPLITING';    % set this to GROWTH or SPLITING 
switch steps
    
    case 'GROWTH'  
  
    for geom=1:n

    string=['cords_',num2str(geom)]; % storing outputs for each geometry
    mkdir(string)
    currentFolder=pwd;

    [coordinates,dimension,numberofcells,walls,wall2cell,l_n,Cellarea,CellvertexId,t,l_n_0,y,epiId,paliId,hypoId,KK]=GROWTH(ap,ar,dr,l_n_max_2_l_n_0,P_h_2_w,geom,numcell,string,currentFolder);
   
    coordinates(:,1)=coordinates(:,1)+abs(min(coordinates(:,1)));
    coordinates(:,2)=coordinates(:,2)+abs(min(coordinates(:,2)));
    
%  assign important matrices for each geometry
     cord{geom}=coordinates;
     CellId{geom}=CellvertexId;
     PID{geom}=paliId;
     EID{geom}=epiId;
     WALL{geom}=walls;
     W2C{geom}=wall2cell;
    end

%% save important matrices for to convert later into comsol geometry
    save cord cord
    save CellId CellId
    save PID PID
    save EID EID
    save WALL WALL
    save W2C W2C

    case 'SPLITING'  

    currentFolder=pwd;

    for geom=1:n
    string=['cords_',num2str(geom)];

    [coordinates,dimension,numberofcells,walls,wall2cell,l_n,Cellarea,CellvertexId,t,l_n_0,y,epiId,paliId,KK,geom,adhesive_wall,split_wall,split_turgor_1,split_turgor_2,...
    left_boundary,right_boundary,az_top,az_top_2,az_bottom,az_bottom_2,con_point,con_point_2,con_point_3]=SPLIT(ap,ar,dr,l_n_max_2_l_n_0,geom,string,currentFolder);
   
    coordinates(:,1)=coordinates(:,1)+abs(min(coordinates(:,1)));
    coordinates(:,2)=coordinates(:,2)+abs(min(coordinates(:,2)));
    
    % assign important matrices for each geometry
     cord{geom}=coordinates;
     CellId{geom}=CellvertexId;
     PID{geom}=paliId;
     EID{geom}=epiId;
     WALL{geom}=walls;
     W2C{geom}=wall2cell;
     cd(currentFolder)

    end


%% save important matrices for to convert later into comsol geometry
    cd(currentFolder)
    mkdir('Splitted_geoms')
    cd('Splitted_geoms')

    save cord cord
    save CellId CellId
    save PID PID
    save EID EID
    save WALL WALL
    save W2C W2C
    
    cd(currentFolder)

end
%% visualizing geometries
figure;
hold on
for j=1:length(CellvertexId)
    
        if ismember(j,epiId)
            patch(coordinates(CellvertexId{j},1),coordinates(CellvertexId{j},2),'r','linewidth',0.01)

        elseif ismember(j,paliId)
            patch(coordinates(CellvertexId{j}(:),1),coordinates(CellvertexId{j}(:),2),'m','linewidth',1)
        else

            patch(coordinates(CellvertexId{j},1),coordinates(CellvertexId{j},2),'g')
            hold on
        end
 end
axis('equal') 
 