% Calculate exposed length of mesophyll cells from comsol geometry;
% Author: Moges Retta, moges.retta@kuleuven.be
% Comsol linked with Matlab is required to run this file.
% Comsol version 5 and Matlab 2017 were used to write this code
% The code expects comsol model file .mph and coordinates of the geometry. 

LmLpali=[];
result=[];
LmLpali_spli=[];
LmLpali_split=[];

load coordinates
model = mphload(mph); % mph is the name of the comsol model file
mphmodel(model);

palisade=model.physics('tds').feature('reac2').selection.named('geom1_csel2_dom').inputEntities(); % 
spongy=model.physics('tds').feature('reac2').selection.named('geom1_csel3_dom').inputEntities(); 
Epi=model.physics('tds').feature('reac2').selection.named('geom1_csel1_dom').inputEntities(); 

for i=1:length(palisade)-1
    dom=palisade(i);
bnd_idx1 = mphgetadj(model, 'geom1', 'boundary', 'domain', dom);
    dom2=palisade(i+1);
bnd_idx2 = mphgetadj(model, 'geom1', 'boundary', 'domain', dom2);
common=intersect(bnd_idx1,bnd_idx2);
result=[result common];
end

h1=model.physics('tds').feature('fl1').selection.named('geom1_csel1_bnd').inputEntities(); % epi+RL
h2=model.physics('tds').feature('fl1').selection.named('geom1_csel2_bnd').inputEntities(); % pali+RL
h3=model.physics('tds').feature('fl1').selection.named('geom1_csel3_bnd').inputEntities(); % spongy+RL
h4=model.physics('tds').feature('fl1').selection.named('geom1_csel4_bnd').inputEntities(); % all except RL

pali_no_int=setdiff(h2,result');
pali_no_int_int_spongy=intersect(h3,pali_no_int);
pali_no_int_no_spongyint=setdiff(pali_no_int,pali_no_int_int_spongy);
pali_no_int_int_epi=intersect(h1,pali_no_int);
pali_no_int_no_spongyint_no_epiint=setdiff(pali_no_int_no_spongyint,pali_no_int_int_epi); % exposed pali+RL

total=union(h1,h2);
total2=union(total,h3);
total3=setdiff(total2,h4); % RL bnd
RLbnd_pali=intersect(total3,pali_no_int_no_spongyint_no_epiint);

exposed_pali=setdiff(pali_no_int_no_spongyint_no_epiint,RLbnd_pali);

model.result.numerical.create('int5', 'IntLine');
model.result.numerical('int5').set('data', 'dset1');
model.result.numerical('int5').selection.set(exposed_pali);
model.result.numerical('int5').set('expr', '1');
model.result.table.create('tbl5', 'Table');
model.result.table('tbl5').comments('Line Integration 5 (1)');
model.result.numerical('int5').set('table', 'tbl5');
model.result.numerical('int5').setResult;
L=model.result.table('tbl5').getTableData(false);
L=L(1,2); % lenght of exposed pali
Leaf_width=max(coordinates(:,1))-min(coordinates(:,1));

Lm_L_pali=str2num(L)/Leaf_width;
LmLpali=[LmLpali;Lm_L_pali];
clear model
% ModelUtil.remove(model);

