% Running file; Author: Moges Retta, moges.retta@kuleuven.be
% Comsol linked with Matlab is required to run this file.
% Comsol version 5 and Matlab 2017 were used to write this code
% The code calls for the function comsol_model. Inputs for comsol_model are 
% geometry information.These are matrices containig indices of cells, 
% vertices and  coordingates for each vertex of a cell. In addition, PID 
% and EID matrices specificy which of the cells belong to palisade and 
% epidermis. Cell ids that are not in EID and PID are then ids for spongy 
% cells.
% cord: Coordinates of the corresponding indices of the vertices
% CellID: Indices of each cell, {cells ID, no. of geometries} 
% PID: indices of palisade cells 
% EID: indices of epidermal cells
% model_name: name of the comsol model
% result_Iinc: the rate of photsynthesis in response to irradiance(Iinc)
% result_Ci: the response of photosynthesis to intercellular CO2 (Ci)

load cord   
load CellId  
load PID     
load EID    

result_Iinc=[];
result_Ci=[];

for j= 1:length(CellId)
    
    model_name=['model',num2str(j)];

    % Initializing matrices to contain information for each
    % geometry in CellID
    coordinates=[];
    CellvertexId=[];   
    epiId=[];          
    paliId=[];
 
    coordinates=cord{j};
    CellvertexId=CellId{j};
    epiId=EID{j};
    paliId=PID{j};

    model=Comsol_model(coordinates,CellvertexId,paliId,epiId,model_name,result_Ci,result_Iinc);
 
end
